SystemOrganization addCategory: #TusLibros!

Bag subclass: #Cart
	instanceVariableNames: 'catalog'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Cart class methodsFor: 'error' stamp: 'msalegas 5/29/2013 20:07'!
productIsNotInTheCatalog
	^'Product not in the catalog'! !

!Cart class methodsFor: 'instance creation' stamp: 'achavez 5/30/2013 20:31'!
withCatalog: aCatalog 
	^self new catalog: aCatalog! !

!Cart methodsFor: 'services' stamp: 'msalegas 5/29/2013 20:37'!
add: aProduct
	(catalog includesKey: aProduct) ifTrue: [super add: aProduct] ifFalse: [ self error: self class productIsNotInTheCatalog]
	! !

!Cart methodsFor: 'creation' stamp: 'msalegas 5/29/2013 19:33'!
catalog: aCatalog 
	catalog := aCatalog! !

Dictionary subclass: #Catalog
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Catalog methodsFor: 'services' stamp: 'achavez 6/9/2013 21:12'!
calculateTotalFor: aCart 
	|price|
	price := 0.
	aCart do: [:product | 
			price := price + self at: product.
		].
	^ price! !

TestCase subclass: #TestCart
	instanceVariableNames: 'interface catalog book user pass merchantProcessor merchantProcessorForStolenCard'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!TestCart methodsFor: 'setup' stamp: 'achavez 6/11/2013 19:24'!
setUp	
	book := 'the hobbit'.
	catalog := Catalog new.
	catalog at: book put: 30.
	interface := WebInterface withCatalog: catalog clock: Time now.
	user := 'Pepe'.
	pass := 'xxx'.
	interface addUser: user pass: pass.
	merchantProcessor := PlugableMerchantProcessor withAction: [  ].
	merchantProcessorForStolenCard := PlugableMerchantProcessor withAction: [self error: PlugableMerchantProcessor stolenCard].
	! !

!TestCart methodsFor: '1 - Cart' stamp: 'achavez 6/9/2013 01:20'!
test01CartShouldBeEmptyWhenCreated
	|cart|
	cart := Cart withCatalog: catalog.
	self assert: cart isEmpty! !

!TestCart methodsFor: '1 - Cart' stamp: 'achavez 6/11/2013 18:43'!
test02CartShouldIcludeBooksWhenABookIsAdded
	|cart |
	cart := Cart withCatalog: catalog.
	cart add: book.
	self assert: (cart includes: book); 
		assert: cart size = 1! !

!TestCart methodsFor: '1 - Cart' stamp: 'achavez 6/9/2013 01:21'!
test03ABookShouldNotBeAddedToCartIfItIsNotInTheCatalog
	|cart|
	catalog := Dictionary new.
	cart := Cart withCatalog: catalog.
	self should: [cart add: book] raise: Error withExceptionDo: [ :anError | self assert: anError messageText = Cart productIsNotInTheCatalog ]. 
	self assert: cart isEmpty
	! !

!TestCart methodsFor: '1 - Cart' stamp: 'achavez 6/9/2013 18:33'!
test04AddABookMultipleTimesMakesTheCartSizeToIncrease
	|cart|
	cart := Cart withCatalog: catalog.	
	cart add: book; add: book.
	self assert: (cart size = 2);
		assert: (cart includes: book);
		assert: (cart asSet size = 1)
	! !

!TestCart methodsFor: '1 - Cart' stamp: 'achavez 6/11/2013 18:47'!
test05CartShouldIncludeAllBooksWhenMultipleBooksAreAdded
	|anotherBook cart|
	anotherBook := 'lotr'.
	catalog at: anotherBook put: 10.
	cart := Cart withCatalog: catalog.
	cart add: book; add: anotherBook.
	self assert: (cart includes: book); 
		assert: (cart includes: anotherBook);
		assert: cart size = 2.! !

!TestCart methodsFor: '2 - Cashier' stamp: 'achavez 6/11/2013 18:54'!
test06CheckOutWithEmptyCartRaisesErrorAndNoSaleIsDoneAndDoesNotUseMerchantProcessorServices
	|cart cashier card|
	cart := Cart withCatalog: catalog.
	cashier := Cashier date: Date today withPriceList: catalog merchantProcessor: merchantProcessor.
	card := CreditCard number: 0001000100010001 owner: 'Pedro Perez' expirationDate: Date today month next.
	self should: [cashier checkout: cart card: card] raise: Error withExceptionDo: [ :anError | self assert: anError messageText = Cashier invalidCheckOut ].
	self assert: cashier lastSale isNil;
		assert:  (merchantProcessor transactionsFor: cashier) isEmpty
	! !

!TestCart methodsFor: '2 - Cashier' stamp: 'achavez 6/9/2013 19:23'!
test07CheckOutWithExpiredCreditCardMakesNoSaleAndRaisesErrorAndDoesNotUseMerchantProcessorServices
	|cart cashier card |
	cart := Cart withCatalog: catalog.
	cart add: book.
	cashier := Cashier date: Date today withPriceList: catalog merchantProcessor: merchantProcessor.
	card := CreditCard number: 0001000100010001 owner: 'Pedro Perez' expirationDate: Date today month - 1.
	self should: [ cashier checkout: cart card: card ] raise: Error withExceptionDo: [ :anError | self assert: anError messageText = Cashier creditCardExpired ]. 
	self assert: cashier lastSale isNil;
		assert:  (merchantProcessor transactionsFor: cashier) isEmpty
	
	
	! !

!TestCart methodsFor: '2 - Cashier' stamp: 'achavez 6/9/2013 19:26'!
test08CheckOutWithStolenCreditCardMakeNoSaleAndMPTransactionFails
	|cart cashier card sale|
	cart := Cart withCatalog: catalog.
	cart add: book.
	cashier := Cashier date: Date today withPriceList: catalog merchantProcessor: merchantProcessorForStolenCard.
	card := CreditCard number: 0001000100010001 owner: 'Pedro Perez' expirationDate: Date today month next.
	self should: [cashier checkout: cart card: card] raise: Error withExceptionDo: [ :anError | self assert: anError messageText = PlugableMerchantProcessor stolenCard ].
	sale := cashier lastSale.
	self assert: sale isNil;
		deny: (merchantProcessorForStolenCard transactionsFor: card) last result
	
		
	
	! !

!TestCart methodsFor: '2 - Cashier' stamp: 'achavez 6/9/2013 19:25'!
test09CheckOutWithValidCreditCardWithMakesASale
	|cart cashier card sale|
	cart := Cart withCatalog: catalog.
	cart add: book.
	cashier := Cashier date: Date today withPriceList: catalog merchantProcessor: merchantProcessor.
	card := CreditCard number: 0001000100010001 owner: 'Pedro Perez' expirationDate: Date today month next.
	cashier checkout: cart card: card.
	sale := cashier lastSale.
	self deny: sale isNil;
		assert: (merchantProcessor transactionsFor: card) last result;
		assert: sale cashier = cashier;
		assert: sale card = card;
		assert: sale priceList = catalog;
		assert: sale products = cart 
		
	
	! !

!TestCart methodsFor: '3 - Client' stamp: 'achavez 6/11/2013 19:24'!
test10CreateACartWithInvalidUserNameRaisesErrorAndDoesNotCreateACart
	|cart anotherUser|
	anotherUser := 'Pancho'.
	self assert: anotherUser ~= user.
	self should: [ cart := interface createCartFor: anotherUser pass: pass ] raise: Error withExceptionDo: [ :anError | self assert: anError messageText = WebInterface invalidUser ].
	self assert: cart isNil; 
		assert: (interface cartFor: user) isNil! !

!TestCart methodsFor: '3 - Client' stamp: 'achavez 6/11/2013 19:24'!
test11CreateACartWithInvalidPasswordRaisesErrorAndDoesNotCreateACart
	|cart|
	self should: [ cart := interface createCartFor: user pass: pass, 'x' ] raise: Error withExceptionDo: [ :anError | self assert: anError messageText = WebInterface invalidPassword ].
	self assert: cart isNil;
		assert: (interface cartFor: user) isNil! !

!TestCart methodsFor: '3 - Client' stamp: 'achavez 6/11/2013 19:13'!
test12CreateACartWithValidUserAndPasswordCreatesACart 
	|session cart|
	session := interface createCartFor: user pass: pass.	
	cart := session cart.
	self deny: cart isNil;
		deny: (interface cartFor: user) isNil;
		assert: cart = (interface cartFor: user);
		assert: cart isEmpty;
		deny: (session isTimedOutAt: interface time)! !

!TestCart methodsFor: '3 - Client' stamp: 'achavez 6/11/2013 19:15'!
test13AddABookToCartWithin30MinutesAddTheBookIndeed
	|session cart|
	session := interface createCartFor: user pass: pass.	
	cart := session cart.
	interface add: book quantity: 1 to: cart.
	self assert: (cart includes: book);
		deny: (session isTimedOutAt: interface time)! !

!TestCart methodsFor: '3 - Client' stamp: 'achavez 6/9/2013 19:58'!
test14AddABookToCartAfter30MinuteSessionFailsAndDoesNotAddTheBook
	|session thirtyMinutesFromNow cart|
	session := interface createCartFor: user pass: pass.	
	thirtyMinutesFromNow := interface time addSeconds: CartSession timeOutInSeconds+1.
	interface time: thirtyMinutesFromNow.
	cart := session cart.
	self assert: cart isEmpty.
	self should: [interface add: book quantity: 1 to: cart] raise: Error withExceptionDo: [ :anError| self assert: anError messageText = interface class timeOutError ].
	self assert: cart isEmpty;
		assert: (session isTimedOutAt: interface time) ! !

!TestCart methodsFor: '3 - Client' stamp: 'achavez 6/9/2013 19:57'!
test15AddABookToCartAndListWithin30MinuteSessionListsCorrectly
	|session cart list|
	session := interface createCartFor: user pass: pass.	
	cart := session cart.
	interface add: book quantity: 10 to: cart.
	list := interface listCart: session id.
	self assert: (list includes: book);
		assert: list size = 10;
		assert: list asSet size = 1;
		deny: (session isTimedOutAt: interface time)! !

!TestCart methodsFor: '3 - Client' stamp: 'achavez 6/9/2013 19:57'!
test16AddABookToCartAndListAfter30MinuteSessionFailsAndDoesNotListsCorrectly
	|session cart thirtyMinutesFromNow list|
	session := interface createCartFor: user pass: pass.	
	cart := session cart.
	interface add: book quantity: 10 to: cart.
	thirtyMinutesFromNow := interface time addSeconds: CartSession timeOutInSeconds+1.
	interface time: thirtyMinutesFromNow.	
	self should: [list := interface listCart: session id] raise: Error withExceptionDo: [ :anError| self assert: anError messageText = interface class timeOutError ].
	self assert: list isNil;	
		assert: (session isTimedOutAt: interface time)
! !

!TestCart methodsFor: '3 - Client' stamp: 'achavez 6/9/2013 21:31'!
test17AddABookToCartAndListPurchasesAfter30MinuteSessionFailesAndDoesNotListsCorrectly
	|session cart purchases expected|
	session := interface createCartFor: user pass: pass.	
	cart := session cart.
	interface add: book quantity: 10 to: cart.
	purchases := interface listPurchasesFor: user password: pass.
	expected := (catalog at: book) *10.
	self assert: ((purchases quantityFor: book) = 10);
		assert: purchases total = expected;
		deny: (session isTimedOutAt: interface time)! !

!TestCart methodsFor: '3 - Client' stamp: 'achavez 6/9/2013 21:34'!
test18AddABookToCartAndListPurchasesAfter30MinuteSessionFailesAndDoesNotListsCorrectly
	|session cart thirtyMinutesFromNow purchases|
	session := interface createCartFor: user pass: pass.	
	cart := session cart.
	interface add: book quantity: 10 to: cart.
	thirtyMinutesFromNow := interface time addSeconds: CartSession timeOutInSeconds+1.
	interface time: thirtyMinutesFromNow.	
	self should: [purchases := interface listPurchasesFor: user password: pass.] raise: Error withExceptionDo: [ :anError| self assert: anError messageText = interface class timeOutError ].
	self assert: purchases isNil;	
		assert: (session isTimedOutAt: interface time)
! !

Object subclass: #CartSession
	instanceVariableNames: 'cart id timestamp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!CartSession class methodsFor: 'instance creation' stamp: 'achavez 6/8/2013 23:11'!
for: aCart id: aNumber timestamp: aTime
	^self new for: aCart id: aNumber timestamp: aTime! !

!CartSession class methodsFor: 'pirvate' stamp: 'achavez 6/9/2013 01:34'!
timeOutInSeconds
	^30*60! !

!CartSession methodsFor: 'accessing' stamp: 'achavez 6/8/2013 23:12'!
cart
	^ cart! !

!CartSession methodsFor: 'creation' stamp: 'achavez 6/9/2013 00:50'!
for: aCart id: aNumber timestamp: aTime
	cart := aCart.
	id := aNumber.
	timestamp := aTime! !

!CartSession methodsFor: 'accessing' stamp: 'achavez 6/9/2013 00:32'!
id
	^ id! !

!CartSession methodsFor: 'testing' stamp: 'achavez 6/9/2013 01:33'!
isTimedOutAt: aTime
	^(timestamp addSeconds: self class timeOutInSeconds) < aTime ! !

!CartSession methodsFor: 'accessing' stamp: 'achavez 6/8/2013 23:32'!
timestamp
	^ timestamp! !

Object subclass: #Cashier
	instanceVariableNames: 'sales priceList date merchantProcessor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Cashier class methodsFor: 'error' stamp: 'achavez 5/30/2013 20:07'!
creditCardExpired
	^'Credit card has expired'! !

!Cashier class methodsFor: 'instance creation' stamp: 'achavez 6/3/2013 20:16'!
date: aDate withPriceList: aPriceList merchantProcessor: aMerchantProcessor 
	 ^self new date: aDate withPriceList: aPriceList merchantProcessor: aMerchantProcessor ! !

!Cashier class methodsFor: 'error' stamp: 'msalegas 5/29/2013 20:23'!
invalidCheckOut
	^'Invalid check out'! !

!Cashier methodsFor: 'services' stamp: 'achavez 6/9/2013 21:54'!
checkout: aCart card: aCard
	| price sale result |
	aCart isEmpty
		ifTrue: [ ^ self error: self class invalidCheckOut ].
	price := priceList calculateTotalFor: aCart.
	(aCard isExpiredOn: date)
		ifTrue: [ ^ self error: self class creditCardExpired ].
	merchantProcessor debit: price from: aCard.
	result := (merchantProcessor transactionsFor: aCard) last result.
	result
		ifFalse: [ ^ self ].
	sale := Sale
		cashier: self
		card: aCard
		products: aCart
		priceList: priceList.
	sales add: sale! !

!Cashier methodsFor: 'creation' stamp: 'achavez 6/3/2013 20:17'!
date: aDate withPriceList: aPriceList merchantProcessor: aMerchantProcessor 
	date := aDate.
	priceList := aPriceList.
	merchantProcessor := aMerchantProcessor.
	sales := OrderedCollection new! !

!Cashier methodsFor: 'accessing' stamp: 'achavez 6/9/2013 21:08'!
lastSale
	^ sales notEmpty ifTrue: [sales last]! !

Object subclass: #CreditCard
	instanceVariableNames: 'number owner expirationDate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!CreditCard class methodsFor: 'instance creation' stamp: 'achavez 5/30/2013 20:10'!
number: aNumber owner: aString expirationDate: aDate 
	^self new number: aNumber owner: aString expirationDate: aDate ! !

!CreditCard methodsFor: 'accessing' stamp: 'achavez 6/5/2013 21:25'!
isExpiredOn: aDate
	^expirationDate < aDate! !

!CreditCard methodsFor: 'creation' stamp: 'achavez 5/30/2013 20:11'!
number: aNumber owner: aString expirationDate: aDate 
	number := aNumber.
	owner := aString.
	expirationDate := aDate.! !

Object subclass: #PlugableMerchantProcessor
	instanceVariableNames: 'action transactions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!PlugableMerchantProcessor class methodsFor: 'error' stamp: 'achavez 6/5/2013 20:14'!
stolenCard
	^'The specified card has been reported stolen'! !

!PlugableMerchantProcessor class methodsFor: 'instance creation' stamp: 'achavez 6/9/2013 01:08'!
withAction: aBlock
	^self new withAction: aBlock! !

!PlugableMerchantProcessor methodsFor: 'services' stamp: 'achavez 6/9/2013 21:49'!
debit: anAmount from: aCard 
	|existing result transaction|
	existing := self transactionsFor: aCard.
	result := true.
	[[action value] on: Error do: [:e| result := false. e pass].
	] ensure: [
		transaction := Transaction amount: anAmount card: aCard result: result.
		existing add: transaction.
		transactions at: aCard put: existing.
		]! !

!PlugableMerchantProcessor methodsFor: 'accessing' stamp: 'achavez 6/5/2013 20:13'!
transactionsFor: aCard 
	^transactions at: aCard ifAbsent: OrderedCollection new! !

!PlugableMerchantProcessor methodsFor: 'creation' stamp: 'achavez 6/9/2013 01:16'!
withAction: aBlock
	action := aBlock.
	transactions := Dictionary new.! !

Object subclass: #Purchase
	instanceVariableNames: 'content total'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Purchase class methodsFor: 'instance creation' stamp: 'achavez 6/9/2013 21:17'!
content: aCart total: aNumber 
	^ self new content: aCart total: aNumber ! !

!Purchase methodsFor: 'creation' stamp: 'achavez 6/9/2013 21:44'!
content: aCart total: aNumber 
	content := aCart.
	total := aNumber! !

!Purchase methodsFor: 'accessing' stamp: 'achavez 6/11/2013 19:20'!
quantityFor: aProduct
	^ content occurrencesOf: aProduct! !

!Purchase methodsFor: 'accessing' stamp: 'achavez 6/9/2013 21:26'!
total
	^ total! !

Object subclass: #Sale
	instanceVariableNames: 'cashier card products priceList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Sale class methodsFor: 'instance creation' stamp: 'achavez 6/3/2013 20:09'!
cashier: aCashier card: aCard products: aCollection priceList: aPriceList
	^self new cashier: aCashier card: aCard products: aCollection priceList: aPriceList! !

!Sale methodsFor: 'accessing' stamp: 'achavez 6/9/2013 02:40'!
card
	^ card! !

!Sale methodsFor: 'accessing' stamp: 'achavez 6/9/2013 02:40'!
cashier
	^ cashier! !

!Sale methodsFor: 'creation' stamp: 'achavez 6/9/2013 01:18'!
cashier: aCashier card: aCard products: aCollection priceList: aPriceList 
	cashier := aCashier.
	card := aCard.
	products := aCollection.
	priceList := aPriceList! !

!Sale methodsFor: 'accessing' stamp: 'achavez 6/9/2013 02:41'!
priceList
	^ priceList! !

!Sale methodsFor: 'accessing' stamp: 'achavez 6/9/2013 02:41'!
products
	^ products! !

Object subclass: #Transaction
	instanceVariableNames: 'amount card result'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Transaction class methodsFor: 'instance creation' stamp: 'achavez 6/3/2013 20:53'!
amount: anAmount card: aCard result: aResult 
	^self new amount: anAmount card: aCard result: aResult! !

!Transaction methodsFor: 'creation' stamp: 'achavez 6/3/2013 21:37'!
amount: anAmount card: aCard result: aResult 
	amount := anAmount.
	card := aCard.
	result := aResult ! !

!Transaction methodsFor: 'accessing' stamp: 'achavez 6/3/2013 21:02'!
result
	^ result! !

Object subclass: #WebInterface
	instanceVariableNames: 'catalog users time sessions'
	classVariableNames: 'Id'
	poolDictionaries: ''
	category: 'TusLibros'!

!WebInterface class methodsFor: 'services' stamp: 'achavez 6/9/2013 21:55'!
id 
	Id ifNil: [ Id := 0 ]. 
	Id := Id + 1.
	^ Id! !

!WebInterface class methodsFor: 'error' stamp: 'achavez 6/5/2013 21:15'!
invalidPassword
	^'Specified user name and password are incorrect'! !

!WebInterface class methodsFor: 'error' stamp: 'achavez 6/5/2013 21:15'!
invalidUser
	^'Specified user name is invalid'! !

!WebInterface class methodsFor: 'error' stamp: 'achavez 6/9/2013 21:47'!
notCartFound
	^'Cart session specified not found'! !

!WebInterface class methodsFor: 'error' stamp: 'achavez 6/8/2013 23:18'!
timeOutError
	^'Session time out'! !

!WebInterface class methodsFor: 'instance creation' stamp: 'achavez 6/8/2013 21:59'!
withCatalog: aCatalog clock: aTime
	^self new withCatalog: aCatalog clock: aTime! !

!WebInterface methodsFor: 'services' stamp: 'achavez 6/9/2013 01:30'!
add: aProduct quantity: aNumber to: aCart 
	|session|
	session := self cartSessionFor: aCart.
	(session isTimedOutAt: time) ifTrue: [ self error: self class timeOutError ].
	aNumber timesRepeat: [aCart add: aProduct]
	
	! !

!WebInterface methodsFor: 'private' stamp: 'achavez 6/9/2013 20:02'!
addUser: user pass: pass 
	users at: user put:  pass! !

!WebInterface methodsFor: 'accessing' stamp: 'achavez 6/9/2013 20:16'!
cartFor: aUserName
	^(sessions at: aUserName ifAbsent: [^nil]) cart.! !

!WebInterface methodsFor: 'accessing' stamp: 'achavez 6/9/2013 20:21'!
cartFromId: aNumber
	|session|
	session := sessions detect: [ :s| s id = aNumber ] ifNone: [^nil].
	^session cart.
! !

!WebInterface methodsFor: 'accessing' stamp: 'achavez 6/9/2013 20:25'!
cartSessionFor: aCart
	^sessions detect: [ :s| s cart = aCart] ifNone: nil! !

!WebInterface methodsFor: 'services' stamp: 'achavez 6/9/2013 20:14'!
createCartFor: aUserName pass: aPassword
	|valid cart session id|
	valid := self validate: aUserName pass: aPassword.
	valid ifTrue: [ 
		cart := Cart withCatalog: catalog. 
		id := self class id.
		session := CartSession for: cart id: id timestamp: time.
		sessions at: aUserName put: session.
		].
	^session! !

!WebInterface methodsFor: 'services' stamp: 'achavez 6/9/2013 01:32'!
listCart: aNumber 
	|session cart|
	session := self cartSessionFor: (self cartFromId: aNumber).
	(session isTimedOutAt: time) ifTrue: [ self error: self class timeOutError ].
	cart := (self cartFromId: aNumber) ifNil: [ ^#() ].
	^cart asArray! !

!WebInterface methodsFor: 'services' stamp: 'achavez 6/9/2013 21:11'!
listPurchasesFor: aUserName password: aPassword 
	|cart session total|
	self validate: aUserName pass: aPassword.
	cart := self cartFor: aUserName.
	session := self cartSessionFor: cart.
	session isNil ifTrue: [ ^self error: self class notCartFound ].
	(session isTimedOutAt: time) ifTrue: [ self error: self class timeOutError ].
	total := catalog calculateTotalFor: session cart.
	^ Purchase content: session cart total: total
	! !

!WebInterface methodsFor: 'accessing' stamp: 'achavez 6/9/2013 19:55'!
time
	^ time! !

!WebInterface methodsFor: 'accessing' stamp: 'achavez 6/9/2013 19:57'!
time: aTime
	time := aTime! !

!WebInterface methodsFor: 'services' stamp: 'achavez 6/5/2013 21:13'!
validate: aUserName pass: aPassword
	|pass|
	pass := users at: aUserName ifAbsent: [ ^self error: self class invalidUser ].
	pass = aPassword ifFalse: [ ^self error: self class invalidPassword ].
	^true
	! !

!WebInterface methodsFor: 'creation' stamp: 'achavez 6/9/2013 20:15'!
withCatalog: aCatalog clock: aTime
	users := Dictionary new.
	sessions := Dictionary new.
	catalog := aCatalog.
	time := aTime! !
